#!/bin/sh
# Copyright start
# Copyright (C) 2008 - 2023 Fortinet Inc.
# All rights reserved.
# FORTINET CONFIDENTIAL & FORTINET PROPRIETARY SOURCE CODE
# Copyright end

# Configuration script for cyops-postgresql

fresh_install(){

    # Start postgres
    systemctl enable postgresql-$ver_new

    # initdb
    if [ ! "$(ls -A /$data_dir_new/)" ]; then
        /usr/pgsql-$ver_new/bin/postgresql-$ver_new-setup initdb
    fi

    # Add our custom configs
    yes | install -o postgres -g postgres /opt/cyops/configs/pgsql/pg_hba.conf $data_dir_new/
    yes | install -o postgres -g postgres /opt/cyops/configs/pgsql/postgresql.conf $data_dir_new/

    # Setup ssl form cyops-certs
    yes | cp /etc/pki/cyops/server.leaf.* /var/lib/pgsql/ssl/
    chmod 600 /var/lib/pgsql/ssl/server.leaf.*
    chown -R postgres: /var/lib/pgsql/ssl/

    systemctl start postgresql-$ver_new
    systemctl status postgresql-$ver_new -l

    sudo -u postgres psql -U postgres -c "DROP USER IF EXISTS cyberpgsql"
    set +x
    echo "sudo -u postgres psql -U postgres -c "CREATE USER cyberpgsql WITH SUPERUSER PASSWORD '*****'""
    sudo -u postgres psql -U postgres -c "CREATE USER cyberpgsql WITH SUPERUSER PASSWORD 'qaplplqa'"
    set -x
}

execute_file()
{
    local file_to_execute=$1
    if [ -f $file_to_execute ];then
        sudo -u postgres ./$file_to_execute
    else
        echo "File [$file_to_execute] not found"
    fi
    return
}

handle_noreplace_files()
{
    file=/opt/cyops/configs/pgsql/postgresql.conf

    # modify file to support upgrade from PG 12 to PG 14

    sudo sed -i "s/^\s*data_directory \+\= \+'\/var\/lib\/pgsql\/12\/data' *$/data_directory = '\/var\/lib\/pgsql\/14\/data'/g"  $file

    sudo sed -i "s/^\s*hba_file \+\= \+'\/var\/lib\/pgsql\/12\/data\/pg_hba\.conf' *$/hba_file = '\/var\/lib\/pgsql\/14\/data\/pg_hba.conf'/g"  $file

    sudo sed -i "s/^\s*ident_file \+\= \+'\/var\/lib\/pgsql\/12\/data\/pg_ident\.conf' *$/ident_file = '\/var\/lib\/pgsql\/14\/data\/pg_ident.conf'/g"  $file
  
    sudo sed -i "s/^\s*external_pid_file \+\= \+'\/var\/run\/12-data\.pid' *$/external_pid_file = '\/var\/run\/14-data.pid'/g"  $file

}

update_memory_files_values(){
    local f_postgres_conf="postgresql.conf"
    local f_postgres_conf_rpmnew="postgresql.conf.rpmnew"
    local d_postgres_conf="/opt/cyops/configs/pgsql/"
    local a_list="shared_buffers temp_buffers max_prepared_transactions work_mem maintenance_work_mem max_stack_depth ssl_cert_file ssl_key_file listen_addresses port"
    local f_cmd_cp="/bin/cp"
    $f_cmd_cp "$d_postgres_conf/$f_postgres_conf" "$d_postgres_conf/$f_postgres_conf.bkp"
    if [ -f "$d_postgres_conf/$f_postgres_conf_rpmnew" ]; then
        for s_var in $a_list;
        do
	         s_to_update_value=$(grep "^$s_var" "$d_postgres_conf/$f_postgres_conf")
	         sed -i "/^$s_var/c\\$s_to_update_value" "$d_postgres_conf/$f_postgres_conf_rpmnew"
        done
        yes | $f_cmd_cp "$d_postgres_conf/$f_postgres_conf_rpmnew" "$d_postgres_conf/$f_postgres_conf"
        echo "Successfully updated the $d_postgres_conf/$f_postgres_conf file"
        return
    fi
    echo "The $d_postgres_conf/$f_postgres_conf_rpmnew file not found."
}
update_pg_hba_conf_for_pg14(){
    #
    # From 7.2.2, postgresql upgrade from 12 to 14 and also password method change from trust to md5.
    # While installing the cyops-auth package it try to connect to database using password if md5 is already in \
    # pg_hba.conf file instead of trust.
    #
    local f_cmd_sed="/usr/bin/sed"
    local s_trust="trust"
    local s_md5="md5"
    local i_psql_version=14
    local f_pg_hba_conf="/var/lib/pgsql/${i_psql_version}/data/pg_hba.conf"
    ${f_cmd_sed} -i "s/${s_trust}/${s_md5}/g" $f_pg_hba_conf
    /usr/bin/systemctl restart postgresql-${i_psql_version}
}

upgrade(){

    # initdb
    if [ ! "$(ls -A $data_dir_new/)" ]; then
        /usr/pgsql-$ver_new/bin/postgresql-$ver_new-setup initdb
    fi

    systemctl stop postgresql-$ver_prev

    # Need to run pg_upgrade as non-root
    # We dont have home dir for postgres user, hence creating it
    home_dir_postgres=/home/postgres
    mkdir $home_dir_postgres
    chown postgres:postgres $home_dir_postgres

    cd $home_dir_postgres
    sudo -u postgres /usr/pgsql-$ver_new/bin/pg_upgrade \
            --old-bindir=/usr/pgsql-$ver_prev/bin/ \
            --new-bindir=/usr/pgsql-$ver_new/bin/ \
            --old-datadir=/var/lib/pgsql/$ver_prev/data/ \
            --new-datadir=$data_dir_new/

    # pg_hha.conf and postgresql.conf are noreplace files. If there is manual change to these
    # files before upgrade, then we need to handle the upgrade changes for it.

    # Currently, we are handling noreplace files to support upgrade from PG 11 to 12
    handle_noreplace_files

    # Setup ssl form cyops-certs
    yes | cp /etc/pki/cyops/server.leaf.* /var/lib/pgsql/ssl/
    chmod 600 /var/lib/pgsql/ssl/server.leaf.*
    chown -R postgres: /var/lib/pgsql/ssl/

    # replace postgresql.conf with new postgresql.conf: Done
    update_memory_files_values
    
    # Add Password for postgres user same as system device uuid.
    s_uuid=$(/bin/csadm license --get-device-uuid)
    sudo -Hiu postgres psql -U postgres -c "ALTER USER postgres WITH ENCRYPTED PASSWORD '$s_uuid';"
    
    # Add our custom configs
    yes | install -o postgres -g postgres /opt/cyops/configs/pgsql/pg_hba.conf $data_dir_new/
    yes | install -o postgres -g postgres /opt/cyops/configs/pgsql/postgresql.conf $data_dir_new/
    
    # change pg_hba.conf replace trust with md5
    update_pg_hba_conf_for_pg14

    systemctl enable postgresql-$ver_new
    systemctl start postgresql-$ver_new

    execute_file analyze_new_cluster.sh
    execute_file delete_old_cluster.sh
}

check_version(){
    for ver in $ver_list; do
        if [ -f "/usr/pgsql-$ver/bin/psql" ];then
            # Old version detected
            # Its an upgrade from old version of cyops
            rm -f $cyops_pg_configured_dir/cyops_pg_${ver}_configured
            ver_prev=$ver
            return 1
        fi
    done

    # Its a fresh install
    return 2
}

#######
# Main
#######
# Space separated list of versions of pg supported by cyops
ver_list="12"
ver_new=$1
data_dir_new=/var/lib/pgsql/$ver_new/data/
cyops_pg_configured_dir=/opt/cyops/configs
cyops_pg_configured=$cyops_pg_configured_dir/cyops_pg_${ver_new}_configured

if [ -f "$cyops_pg_configured" ];then
    # Same version is installed.
    # Its probably upgrade of same cyops version
    # No op
    echo "Found $cyops_pg_configured"
    echo "Looks like postgresql $ver_new is already configured"
    echo "Skipping the configuration"
    exit 0
fi

common(){
	. /opt/cyops/adm/transaction.sh
    PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql -d postgres -c "CREATE EXTENSION IF NOT EXISTS hstore;"
    PGPASSWORD=${s_pg_pwd} psql -U cyberpgsql -d postgres -c 'CREATE EXTENSION IF NOT EXISTS "uuid-ossp";'
    touch $cyops_pg_configured
}

# Detect whether it is fresh install or an upgrade
check_version
if [ 2 = $? ];then
    fresh_install
elif [ 1 = $? ];then
    upgrade
fi
common
exit 0
